package com.example.nan_map;

import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.DefaultLifecycleObserver;
import androidx.lifecycle.LifecycleOwner;
import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationListener;
import com.amap.api.maps.AMap;
import com.amap.api.maps.CameraUpdateFactory;
import com.amap.api.maps.LocationSource;
import com.amap.api.maps.MapView;
import com.amap.api.maps.model.CameraPosition;
import com.amap.api.maps.model.LatLng;
import com.amap.api.services.core.LatLonPoint;
import com.amap.api.services.geocoder.GeocodeSearch;
import com.amap.api.services.geocoder.RegeocodeQuery;
import org.jetbrains.annotations.NotNull;
import java.util.Map;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.platform.PlatformView;

public class NanMapView implements LocationSource, AMapLocationListener,AMap.OnCameraChangeListener, PlatformView, DefaultLifecycleObserver, ActivityPluginBinding.OnSaveInstanceStateListener {

    private View myView;
    MapView mapView;
    private AMap aMap;
    private GeocodeSearch geocoderSearch;
    private Context mContext;
    private LatLng latLng;
    private AMapLocationClient mlocationClient;
    private LocationSource.OnLocationChangedListener mListener;
    private AMapLocationClientOption mLocationOption;
    private TextView tvAddress;

    NanMapView(Context context, int viewId, Object args, BinaryMessenger binaryMessenger, LifecycleProvider lifecycleProvider){
        LayoutInflater mInflater = LayoutInflater.from(context);
        myView = mInflater.inflate(R.layout.map_view,null);
        this.mContext = context;
        mapView = myView.findViewById(R.id.mapId);
        lifecycleProvider.getLifecycle().addObserver(this);
        tvAddress =myView.findViewById(R.id.topTextViewId);
        Map map = (Map) args;
        tvAddress.setVisibility("1".equals(map.get("isHidden"))?View.VISIBLE:View.GONE);
        NanMessageChannel.init(context, binaryMessenger, new LifecycleProvider.CurrentPositionAction() {
            @Override
            public void onLocatin() {
                if (mlocationClient != null) {
                    mlocationClient.startLocation();
                }
            }
            @Override
            public void setMapType(int mapType) {
                aMap.setMapType(mapType);
            }
        },viewId);
        if (aMap == null){
            aMap = mapView.getMap();
            setUpMap();
        }
    }

    private void setUpMap(){
        aMap.setLocationSource(this);// 设置定位监听
        aMap.getUiSettings().setMyLocationButtonEnabled(false);// 设置默认定位按钮是否显示
        aMap.setMyLocationEnabled(true);// 设置为true表示显示定位层并可触发定位，false表示隐藏定位层并不可触发定位，默认是false
        // aMap.setMyLocationType()
        aMap.getUiSettings().setZoomControlsEnabled(false);
        aMap.getUiSettings().setZoomPosition(60);
        aMap.moveCamera(CameraUpdateFactory.zoomTo(18));
        aMap.setOnCameraChangeListener(this);
//        aMap.setMapType(AMap.MAP_TYPE_SATELLITE);
        NanMessageChannel.geocoderSearch(new LifecycleProvider.OnLocationAction() {
            @Override
            public void onLocation(String address) {
                tvAddress.setText(address);
            }

        });
    }

    @Override
    public View getView() {
        return myView;
    }

    @Override
    public void dispose() {
        destroyMapViewIfNecessary();
    }

    @Override
    public void onCreate(@NonNull @NotNull LifecycleOwner owner) {
        if (null != mapView){
            mapView.onCreate(null);
        }
    }

    @Override
    public void onStart(@NonNull @NotNull LifecycleOwner owner) {

    }

    @Override
    public void onStop(@NonNull @NotNull LifecycleOwner owner) {

    }

    @Override
    public void onResume(@NonNull @NotNull LifecycleOwner owner) {
        if (null != mapView){
            mapView.onResume();
        }
    }

    @Override
    public void onPause(@NonNull @NotNull LifecycleOwner owner) {
        if (null != mapView){
            mapView.onPause();
        }
    }

    @Override
    public void onDestroy(@NonNull @NotNull LifecycleOwner owner) {
        destroyMapViewIfNecessary();
    }

    @Override
    public void onSaveInstanceState(@NonNull @NotNull Bundle bundle) {
        mapView.onSaveInstanceState(bundle);
    }

    @Override
    public void onRestoreInstanceState(@Nullable @org.jetbrains.annotations.Nullable Bundle bundle) {
        mapView.onCreate(bundle);
    }

    private void destroyMapViewIfNecessary() {
        if (mapView == null) {
            return;
        }
        mapView.onDestroy();
    }

    @Override
    public void onLocationChanged(AMapLocation aMapLocation) {
        if (mListener != null && aMapLocation != null) {
            if (aMapLocation != null
                    && aMapLocation.getErrorCode() == 0) {
                mListener.onLocationChanged(aMapLocation);// 显示系统小蓝点
                latLng = new LatLng(aMapLocation.getLatitude(),aMapLocation.getLongitude());
                aMap.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, 19));
            } else {
                String errText = "定位失败," + aMapLocation.getErrorCode() + ": " + aMapLocation.getErrorInfo();
                Log.e("AmapErr", errText);
            }
        }
    }

    @Override
    public void activate(OnLocationChangedListener onLocationChangedListener) {
        mListener = onLocationChangedListener;
        if (mlocationClient == null) {
            mlocationClient = new AMapLocationClient(mContext);
            mLocationOption = new AMapLocationClientOption();
            //设置定位监听
            mlocationClient.setLocationListener(this);
            //设置为高精度定位模式
            mLocationOption.setLocationMode(AMapLocationClientOption.AMapLocationMode.Hight_Accuracy);
//            mLocationOption.setInterval(1000 * 60 * 60);
//            mLocationOption.setInterval(60 * 1000);//每10秒定位一次
            //使用连续定位
            mLocationOption.setOnceLocation(true);
            //设置定位参数
            mlocationClient.setLocationOption(mLocationOption);
            // 此方法为每隔固定时间会发起一次定位请求，为了减少电量消耗或网络流量消耗，
            // 注意设置合适的定位时间的间隔（最小间隔支持为2000ms），并且在合适时间调用stopLocation()方法来取消定位请求
            // 在定位结束后，在合适的生命周期调用onDestroy()方法
            // 在单次定位情况下，定位无论成功与否，都无需调用stopLocation()方法移除请求，定位sdk内部会移除
            mlocationClient.startLocation();
        }
    }

    @Override
    public void deactivate() {
        mListener = null;
        if (mlocationClient != null) {
            mlocationClient.stopLocation();
            mlocationClient.onDestroy();
        }
        mlocationClient = null;
    }

    @Override
    public void onCameraChange(CameraPosition cameraPosition) {

    }

    @Override
    public void onCameraChangeFinish(CameraPosition cameraPosition) {
        LatLng position = cameraPosition.target;//screenMarker.getPosition();
        LatLonPoint latLonPoint = new LatLonPoint(position.latitude, position.longitude);
        RegeocodeQuery query = new RegeocodeQuery(latLonPoint, 200,
                GeocodeSearch.AMAP);// 第一个参数表示一个Latlng，第二参数表示范围多少米，第三个参数表示是火系坐标系还是GPS原生坐标系
//        geocoderSearch.getFromLocationAsyn(query);// 设置异步逆地理编码请求
        NanMessageChannel.geocoderSearch.getFromLocationAsyn(query);// 设置异步逆地理编码请求
    }
}
